%{
// flex -+ flextest.l; c++ -O2 lex.yy.cc
// flex -+ --full flextest.l; c++ -O2 lex.yy.cc
#include "reflex/timer.h"
#include <cstdio>
#include <fstream>
%}

%o nodefault

WHITESPACE	[[:space:]]+
ILCOMMENT	"//"([^\n]|\\\n)*\n
MLCOMMENT	"/*"([^*]|\*+[^/])*"*/"
DIRECTIVE	^[[:blank:]]*#(.|\\\r?\n)+
NAME		[[:alpha:]_][[:alnum:]_]*
FLOAT		[-+][0-9]*([0-9]|\.[0-9]|[0-9]\.)[0-9]*([Ee][-+]?[0-9]+)?[FfLl]?
UOCT		0[0-7]+
UHEX		0[Xx][[:xdigit:]]+
UDEC		0|[1-9][0-9]*
INT		[-+]?({UOCT}|{UHEX}|{UDEC})([Ll]{0,2}[Uu]?|[Uu][Ll]{0,2})
CHAR		L?'(\\.|[^\\\n'])*'
STRING		L?\"(\\.|\\\r?\n|[^\\\n"])*\"

%%

{WHITESPACE}	|
{ILCOMMENT}	|
{MLCOMMENT}	|
{DIRECTIVE}	|
{NAME}		|
{FLOAT}		|
{INT}		|
{CHAR}		|
{STRING}	|
"+="		|
"++"		|
"+"		|
"-="		|
"--"		|
"->"		|
"-"		|
"=="		|
"="		|
"<="		|
"<<="		|
"<<"		|
"<"		|
">="		|
">>="		|
">>"		|
">"		|
"!="		|
"!"		|
","		|
";"		|
"..."		|
"."		|
"^="		|
"^"		|
"~"		|
"*="		|
"*"		|
"/="		|
"/"		|
"%="		|
"%"		|
"&="		|
"&&"		|
"&"		|
"|="		|
"||"		|
"|"		|
"("		|
")"		|
"["		|
"]"		|
"{"		|
"}"		|
"::"		|
":"		|
"?"		return 1;
.		printf("*** ERROR '%s' at line %d\n", yytext, yylineno);

%%

int yyFlexLexer::yywrap() { return 1; }

#define RUNS 10000

int main()
{
  yyFlexLexer lexer;

  std::ifstream file("re2test.cpp");
  // std::string str((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
  // file.close();

  size_t n = 0;
  reflex::timer_type t;
  reflex::timer_start(t);
  for (int run = 0; run < RUNS; ++run)
  {
    file.clear(); // always do this before seekg()
    file.seekg(0);
    lexer.yyrestart(&file);
    n = 0;
    while (lexer.yylex())
    {
      ++n;
      // std::cout << lexer.text() << std::endl;
    }
  }
  float dt = reflex::timer_elapsed(t);
  std::cout << n << " matches, elapsed " << 1000*dt/RUNS << " micro seconds" << std::endl;

  file.close();
}
