// bin/reflex reflextest.l; c++ -O2 -Iinclude -I/opt/local/include lex.yy.cpp lib/libreflex.a /opt/local/lib/libboost_regex-mt.dylib
%top{
#include <iostream>
#include <fstream>
#include <reflex/timer.h>
%}

// no lazy quants (not supported by Boost.Regex POSIX mode):
// %o matcher=boost-perl
// %o matcher=boost
// std-ecma requires removal of ^ from DIRECTIVE
// %o matcher=std-ecma
WHITESPACE	[[:space:]]+
ILCOMMENT	"//"([^\n]|\\\n)*\n
MLCOMMENT	"/*"([^*]|\*+[^/])*"*/"
DIRECTIVE	^[[:blank:]]*#(.|\\\r?\n)+
NAME		[[:alpha:]_][[:alnum:]_]*
FLOAT		[-+]\d*(\d|\.\d|\d\.)\d*([Ee][-+]?\d+)?[FfLl]?
UOCT		0[0-7]+
UHEX		0[Xx][[:xdigit:]]+
UDEC		0|[1-9]\d*
INT		[-+]?({UOCT}|{UHEX}|{UDEC})([Ll]{0,2}[Uu]?|[Uu][Ll]{0,2})
CHAR		L?'(\\.|[^\\\n'])*'
STRING		L?\"(\\.|\\\r?\n|[^\\\n"])*\"

%o flex
%o nodefault

%%

{WHITESPACE}	|
{ILCOMMENT}	|
{MLCOMMENT}	|
{DIRECTIVE}	|
{NAME}		|
{FLOAT}		|
{INT}		|
{CHAR}		|
{STRING}	|
"+="		|
"++"		|
"+"		|
"-="		|
"--"		|
"->"		|
"-"		|
"=="		|
"="		|
"<="		|
"<<="		|
"<<"		|
"<"		|
">="		|
">>="		|
">>"		|
">"		|
"!="		|
"!"		|
","		|
";"		|
"..."		|
"."		|
"^="		|
"^"		|
"~"		|
"*="		|
"*"		|
"/="		|
"/"		|
"%="		|
"%"		|
"&="		|
"&&"		|
"&"		|
"|="		|
"||"		|
"|"		|
"("		|
")"		|
"["		|
"]"		|
"{"		|
"}"		|
"::"		|
":"		|
"?"		return 1;
.		printf("*** ERROR '%s' at line %d\n", yytext, yylineno);

%%

#define RUNS 10000

int main()
{
  yyFlexLexer lexer;

  std::ifstream file("re2test.cpp");
  std::string str((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
  file.close();

  size_t n = 0;
  reflex::timer_type t;
  reflex::timer_start(t);
  for (int run = 0; run < RUNS; ++run)
  {
    lexer.in(str);
    n = 0;
    while (lexer.yylex())
    {
      ++n;
      // std::cout << lexer.text() << std::endl;
    }
  }
  float dt = reflex::timer_elapsed(t);
  std::cout << n << " matches, elapsed " << 1000*dt/RUNS << " micro seconds" << std::endl;
}
