
// https://www.gnu.org/software/libc/manual/html_node/Regular-Expressions.html#Regular-Expressions
// removed (?m)
// removed (?:X) non-capturing groups
// removed and replaced all escapes such as \s, \Q, \E, \d, \n, \r
// replaced \\ by [\\]
// c++ -O2 regextest.cpp

#include <regex.h>
#include <iostream>
#include <fstream>
#include <cassert>
#include "reflex/timer.h"

#define RUNS 1000

int main()
{
  const char *regex = "([ \t\n]+)|(//(.|[\\]\n)*\n)|((/\\*)([^*]|\\*+[^/])*\\*/)|([ \t]*#(.|[\\]\r?\n)+)|([A-Z_a-z][0-9A-Z_a-z]*)|([+-][0-9]*([0-9]|\\.[0-9]|[0-9]\\.)[0-9]*([Ee][+-]?[0-9]+)?[FLfl]?)|([+-]?((0[0-7]+)|(0[Xx][0-9A-Fa-f]+)|(0|[1-9][0-9]*))([Ll]{0,2}[Uu]?|[Uu][Ll]{0,2}))|(L?'([\\]\\.|[^\n'\\])*')|(L?\"([\\].|[\\][\\]\r?\n|[^\n\"\\])*\")|(\\+=)|(\\+\\+)|(\\+)|(-=)|(--)|(->)|(-)|(==)|(=)|(<=)|(<<=)|(<<)|(<)|(>=)|(>>=)|(>>)|(>)|(!=)|(!)|(,)|(;)|(\\.\\.\\.)|(\\.)|(\\^=)|(\\^)|(~)|(\\*=)|(\\*)|(/=)|(/)|(%=)|(%)|(&=)|(&&)|(&)|(\\|=)|(\\|\\|)|(\\|)|(\\()|(\\))|(\\[)|(\\])|(\\{)|(\\})|(::)|(:)|(\\?)";
  regex_t pattern;
  int ret = regcomp(&pattern, regex, REG_EXTENDED | REG_NEWLINE);
  if (ret)
  {
    char buf[1024];
    regerror(ret, &pattern, buf, sizeof(buf));
    printf("Error: %s\n", buf);
    exit(1);
  }

  std::ifstream file("re2test.cpp");
  std::string str((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
  file.close();

  const char *subject = str.c_str();
  regmatch_t match;

  size_t n = 0;
  reflex::timer_type t;
  reflex::timer_start(t);
  for (int run = 0; run < RUNS; ++run)
  {
    n = 0;
    size_t offset = 0;
    while (regexec(&pattern, subject + offset, 1, &match, REG_NOTBOL) == 0)
    {
      // printf("match %c%c from %lld to %lld\n", subject[offset + match.rm_so], subject[offset + match.rm_so + 1], match.rm_so, match.rm_eo);
      offset += match.rm_eo;
      ++n;
    }
  }
  float dt = reflex::timer_elapsed(t);
  std::cout << n << " matches, elapsed " << 1000*dt/RUNS << " micro seconds" << std::endl;
}
