// http://www.boost.org/doc/libs/1_63_0/libs/spirit/doc/html/spirit/lex/tutorials/lexer_quickstart1.html
// Documentation incorrectly states that . matches any except \n: it matches any and \n

// c++ -O2 -I. -I/opt/local/include spirittest.cpp
 
#include <iostream>
#include <fstream>
#include <boost/spirit/include/lex_lexertl.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include "reflex/timer.h"

#define RUNS 10000

namespace lex = boost::spirit::lex;

template <typename Lexer>
struct c_tokens : lex::lexer<Lexer>
{
  c_tokens()
  {
    this->self.add
      ("\\s+", 1)
      ("\"//\"([^\\n]|\\\\\\n)*\\n", 2)
      ("\"/*\"([^*]|\\*+[^/])*\"*/\"", 3)
      ("^[ \\t]*#([^\\n]|\\\\\\r?\\n)+", 4)
      ("[A-Za-z_][0-9A-Za-z_]*", 5)
      ("[-+][0-9]*([0-9]|\\.[0-9]|[0-9]\\.)[0-9]*([Ee][-+]?[0-9]+)?[FfLl]?", 6)
      ("[-+]?(0|[1-9][0-9]*|0[Xx][0-9A-Fa-f]+|0[0-7]+)([Ll]{0,2}[Uu]?|[Uu][Ll]{0,2})", 7)
      ("L?'(\\\\.|[^\\\\\\n'])*'", 8)
      ("L?\\\"(\\\\.|\\\\\\r?\\n|[^\\\\\\n\"])*\\\"", 9)
      ("\"+=\"", 10)
      ("\"++\"", 11)
      ("\"+\"", 12)
      ("\"-=\"", 13)
      ("\"--\"", 14)
      ("\"->\"", 15)
      ("\"-\"", 16)
      ("\"==\"", 17)
      ("\"=\"", 18)
      ("\"<=\"", 19)
      ("\"<<=\"", 20)
      ("\"<<\"", 21)
      ("\"<\"", 22)
      ("\">=\"", 23)
      ("\">>=\"", 24)
      ("\">>\"", 25)
      ("\">\"", 26)
      ("\"!=\"", 27)
      ("\"!\"", 28)
      ("\",\"", 29)
      ("\";\"", 30)
      ("\"...\"", 31)
      ("\".\"", 32)
      ("\"^=\"", 33)
      ("\"^\"", 34)
      ("\"~\"", 35)
      ("\"*=\"", 36)
      ("\"*\"", 37)
      ("\"/=\"", 38)
      ("\"/\"", 39)
      ("\"%=\"", 40)
      ("\"%\"", 41)
      ("\"&=\"", 42)
      ("\"&&\"", 43)
      ("\"&\"", 44)
      ("\"|=\"", 45)
      ("\"||\"", 46)
      ("\"|\"", 47)
      ("\"(\"", 48)
      ("\")\"", 49)
      ("\"[\"", 50)
      ("\"]\"", 51)
      ("\"{\"", 52)
      ("\"}\"", 53)
      ("\"::\"", 54)
      ("\":\"", 55)
      ("\"?\"", 56)
      (".", 57)
      ;
  }
};

int main()
{
  std::ifstream file("re2test.cpp");
  std::string str((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
  file.close();

  typedef lex::lexertl::token<char const*, lex::omit, boost::mpl::false_> token_type;

  typedef lex::lexertl::actor_lexer<token_type> lexer_type;

  c_tokens<lexer_type> tokenizer;

  size_t n = 0;
  reflex::timer_type t;
  reflex::timer_start(t);
  for (int run = 0; run < RUNS; ++run)
  {
    char const *first = str.c_str(), *last = &first[str.size()];
    lexer_type::iterator_type iter, end = tokenizer.end();
    n = 0;
    for (iter = tokenizer.begin(first, last); iter != end && token_is_valid(*iter); ++iter)
    {
      ++n;
      // std::cout << *iter << std::endl;
    }
  }
  float dt = reflex::timer_elapsed(t);
  std::cout << n << " matches, elapsed " << 1000*dt/RUNS << " micro seconds" << std::endl;
}
